# Application
#
# Main application instance of a Shot application. Inherits from EventdServer and implements various
# "routers", which take care of handling special Shot emissions, such as controller calling, etc.
#
# Shot Framework - Copyright (c) Jesse Aaron Dunlap <me@jessedunlap.me>
# Licensed under the MIT License. For full licensing information, please
# see LICENSE.md. http://github.com/JesseDunlap/shot/

require 'eventd'

require_relative './application_instance'

class Application < EventdServer
	# Array of routers
	attr_accessor :routers

	# Array of loaders
	attr_accessor :loaders

	# Active connected clients
	attr_accessor :clients

	# Temporary data for storage of state-based information
	attr_accessor :data

	# Initialize the Application and EventdServer. Specifies a default configuration which
	# allows the EventdServer to bind on http://127.0.0.1:8080

	def initialize(options = { :host => '127.0.0.1', :port => 8080 })
		super(options)
		@routers = []
		@loaders = []
		@clients = []
		@data = {}
		setup_handlers
	end

	# Add a router to the list of custom routers the application supports.
	#
	# Note:: Existing clients will not be handled by the new router.
	#
	# ==Attributes
	#
	# * +router+ - A class which inherits from Router

	def add_router(router)
		@routers.push router unless @routers.include? router
	end

	# Add a loader to the list of custom loaders the application supports.
	#
	# == Attributes
	#
	# * +loader+ - A class which inherits from Loader

	def add_loader(loader)
		unless @loaders.include? loader
			@loaders.push loader

			@clients.each do |client|
				client.add_loader loader
			end
		end
	end

	# Gets a value from the state-based store

	def get(key)
		@data[key]
	end

	# Sets a value to the state-based store

	def set(key, value)
		@data[key] = value
	end

	private

	def setup_handlers
		self.on 'connection' do |client|
			client.on 'connect' do
				application_instance = ApplicationInstance.from_eventd_client client, self

				@clients.push application_instance

				@loaders.each do |loader|
					application_instance.add_loader loader
				end

				@routers.each do |router|
					router.new(application_instance)
				end

				self.emit 'load', application_instance
			end
		end
	end
end
