# ApplicationInstance
#
# Inherits from EventdClient and represents a single client in a Shot Application.
#
# Shot Framework - Copyright (c) Jesse Aaron Dunlap <me@jessedunlap.me>
# Licensed under the MIT License. For full licensing information, please
# see LICENSE.md. http://github.com/JesseDunlap/shot/

require 'eventd'

class ApplicationInstance < EventdClient
	# Array of loaders
	attr_accessor :loaders

	# Main Application instance
	attr_accessor :app

	# App Configuration
	attr_accessor :config

	# Initialize a new ApplicationInstance from an existing EventdClient.
	#
	# Note:: This is the preferred way to initialize an ApplicationInstance.
	#
	# == Attributes
	#
	# * +client+ - EventdClient to copy properties from.

	def self.from_eventd_client(client, app)
		instance = ApplicationInstance.new
		instance.app = app
		instance.socket = client.socket
		instance.setup_socket
		instance
	end

	# Initialize the superclass.
	#
	# Note::
	#   Please use the ApplicationInstance#from_eventd_client method instead, since it is not usually
	#   a good idea to instantiate an EventdClient without a socket.

	def initialize(socket = nil)
		super(socket)
		@config = {}
		@loaders = {}
	end

	# Add a loader to the list of custom loaders the application supports.
	#
	# == Attributes
	#
	# * +loader+ - A class which inherits from Loader

	def add_loader(loader)
		loader_instance = loader.new self
		@loaders[loader_instance.type] = loader_instance
	end

	# Calls out to any available loader and loads the specified file

	def get(type, file)
		if @loaders[type]
			@loaders[type].get file
		else
			raise LoaderNotFoundException.new "Could not load type #{type}"
		end
	end
end